FROM golang:1.22-alpine3.18

ENV CGO_ENABLED=0
ENV GOOS=linux

# Add app user
RUN adduser --no-create-home -u 1000 -D appuser


# O problema é que na primeira vez que o código é compilado, demora muito (mais de 5 segundos que é o timeout)
# A ideia aqui é executar um programa teste, e então copiar o cache quando tiver o container executando
RUN <<EOF
echo 'package main' >> /example.go
echo 'import (' >> /example.go
echo '  "fmt"' >> /example.go
echo '  "os"' >> /example.go
echo ')' >> /example.go
echo 'func main() {' >> /example.go
echo '  fmt.Println("GOLANG - Enviou:", os.Args)' >> /example.go
echo '}' >> /example.go
EOF

# Grava o cache no /golang, para registrar na imagem
ENV GOTMPDIR=/golang
ENV TMPDIR=/golang
ENV GOCACHE=/golang

RUN mkdir /golang && go run /example.go

# Volta o valor das variáveis para o que deve ser
# https://www.docker.com/blog/introduction-to-heredocs-in-dockerfiles/
# A ideia é a seguinte, para executar um arquivo golang em um container com a flag --read-only
# É necessário utilizar o diretório /dev para qualquer tipo de arquivo temporário usado ao compilar o código
# Funciona pois o /dev é um mount tmpfs que não possui a limitação de apenas leitura do /
# E já que no docker o único processo é o nosso, mexer nesse diretório assim não deveria causar problemas...
# Obs: deve ter permissões para executar.
# (Veja o issue https://github.com/golang/go/issues/8451)
ENV GOTMPDIR=/dev/golang
ENV TMPDIR=/dev/golang
ENV GOCACHE=/dev/golang

# Configure working directory
WORKDIR /code
RUN chown -R appuser /code

ENTRYPOINT [ "/bin/sh", "-c" ]

# Irá copiar o cache do golang para o /dev/golang, que só existe enquanto o container está executando
CMD [ "cp -r /golang /dev/golang && sleep 10m" ]