import paginate from "mongoose-paginate-v2";
import mongoose, { SchemaOptions } from "mongoose";

// https://mongoosejs.com/docs/guide.html#options
export const defaultSchemaOptions = <T>(): SchemaOptions<T> => ({
    // Para não ter o virtual id
    id: false,
    // https://mongoosejs.com/docs/api/document.html#Document.prototype.toObject()
    toJSON: { virtuals: true },
    toObject: { virtuals: true },
    // Dar erro se tentar salvar um documento com um campo que não está no schema
    strict: "throw"
});

export interface SchemaBase {
    _id?: mongoose.Types.ObjectId;
}

export const applyMongooseInitialConfig = () => {
    paginate.paginate.options = {
        leanWithId: false,
        page: 1,
        limit: 10,
        customLabels: {
            totalPages: false,
            pagingCounter: false,
            hasPrevPage: false,
            hasNextPage: false,
            prevPage: false,
            nextPage: false,
            meta: false       
        }
    };
};