import { getPaginatedResponseSchema, getResponseSchema } from "../utils/messages.js";
import { ProblemaInteractor } from "../interactors/problemaInteractor.js";
import { ProblemaSchemas } from "../schemas/problemaSchemas.js";
import { APIError } from "../middlewares/errorHandler.js";
import ControllerBase, { ParsedSchema, route } from "./ControllerBase.js";

export default class ProblemaController extends ControllerBase {
    public interactor: ProblemaInteractor;
    constructor(
        interactor = new ProblemaInteractor(),
    ) {
        super("Problema");

        this.interactor = interactor;
    }
 
    @route({
        method: "post",
        route: "/problemas",
        schema: ProblemaSchemas.cadastrarProblema,
        response: {code: 201, json: getResponseSchema(ProblemaSchemas.problemaGET)}
    })
    async cadastrarProblema({body}: ParsedSchema<typeof ProblemaSchemas.cadastrarProblema>) {
        const problema = await this.interactor.cadastrarProblema(body);

        return {code: 201, json: problema};
    }

    @route({
        method: "get",
        route: "/problemas",
        schema: ProblemaSchemas.listarProblemas,
        response: {code: 200, json: getPaginatedResponseSchema(ProblemaSchemas.problemaGET)}
    })
    async listarProblemas({query}: ParsedSchema<typeof ProblemaSchemas.listarProblemas>) {
        const result = await this.interactor.repo.listar(query);

        return {code: 200, json: result};
    }

    @route({
        method: "get",
        route: "/problemas/:sigla",
        schema: ProblemaSchemas.obterProblemaPelaSigla,
        response: {code: 200, json: getResponseSchema(ProblemaSchemas.problemaGET)}
    })
    async obterProblemaPelaSigla({params}: ParsedSchema<typeof ProblemaSchemas.obterProblemaPelaSigla>) {
        const problema = await this.interactor.repo.obterPorSigla(params.sigla);
        if (!problema) {
            throw new APIError("Problema não encontrado", 404);
        }

        return {code: 200, json: problema};
    }
}