
import { APIError } from "../middlewares/errorHandler.js";
import { Problema, ProblemaDoc, ProblemaRepository, ProblemaTeste } from "../models/Problema.js";
import sanitizarHTML, { extrairApenasTexto } from "../utils/sanitizarHTML.js";
import { gerarSlugUnico } from "../utils/slugUnico.js";
import { Optional } from "../utils/utilityTypes.js";

export class ProblemaInteractor {
    public repo: ProblemaRepository;

    constructor(
        repository = ProblemaRepository.getInstance()
    ) {
        this.repo = repository;
    }

    async cadastrarProblema(body: {
        sigla?: string,
        titulo: string, 
        descricao: string, 
        testes: Optional<ProblemaTeste, "num">[]
        testes_privados: Optional<ProblemaTeste, "num">[]
    }) {
        let {descricao, testes, testes_privados, sigla, ...rest} = body;
        let newBody: Optional<Problema, "sigla"> = {...rest, ...{
            descricao: "", 
            descricao_html: "", 
            testes: [], 
            testes_privados: [],
            testes_quantidade: 0
        }};

        for(const teste of testes) {
            newBody.testes.push({...teste, ...{num: newBody.testes_quantidade}});
            newBody.testes_quantidade++;
        }
        for(const teste of testes_privados) {
            newBody.testes_privados!.push({...teste, ...{num: newBody.testes_quantidade}});
            newBody.testes_quantidade++;
        }

        // A FAZER: questão do rich-text editor, verificar se está funcionando
        newBody.descricao_html = sanitizarHTML(descricao);
        newBody.descricao = extrairApenasTexto(newBody.descricao_html);

        if(!newBody.descricao || !newBody.descricao_html) {
            throw new APIError("O conteúdo que foi enviado não possui texto algum");
        }
 
        // Se especificar sigla é PUT exigindo exisitir, se não expecificar sigla é porque deve criar
        // DEPOIS: permissões?
        let ret: ProblemaDoc;
        if(sigla !== undefined) {
            let encontrado = await this.repo.obterPorSigla(sigla);
            if(!encontrado)
                throw new APIError("Problema não encontrado", 404);

            ret = await this.repo.patch(encontrado, newBody);
        } else {
            // gerar sigla única segura para urls
            newBody.sigla = await gerarSlugUnico(newBody.titulo, async (str) => {
                return await this.repo.obterPorSigla(str) !== null;
            });
            ret = await this.repo.create(newBody);
        }

        return this.repo.omitUnselected(ret);
    }
}