import { RequestHandler } from "express";
import { APIError } from "./errorHandler.js";
import { decodeJWTToken } from "../utils/decodeToken.js";

const AuthMiddleware: (auth: "require" | "optional") => RequestHandler = 
    (auth) => (req, res, next) => {
        try {
            const bearerToken = req.headers.authorization;

            if (!bearerToken) {
                if(auth === "require") throw new APIError("É necessário se autenticar", 498);
                
                next();
                return;
            }

            // 'Bearer AAAA.BBBB.CCCC' -> ['Bearer', 'AAAA.BBBB.CCCC']
            const [_bearer, token] = bearerToken.split(" ");

            res.locals.token = decodeJWTToken(token);
            next();
        } catch (error) {
            next(error);
        }
    };

export default AuthMiddleware;
