import { z } from "zod";
import { validarForcaSenha } from "../utils/forcaSenha.js";
import { UsuarioSchemas } from "./usuarioSchemas.js";

export class AuthSchemas {
    static fazerLogin = {
        body: z.object({
            email: z.string().email(),
            senha: z.string().min(1).max(255)
        })
    };

    static redefinirSenha = {
        body: z.object({
            senha: z.string(),
            senha_nova: z.string().min(1).max(255).superRefine(validarForcaSenha)
        })
    };

    static recuperarSenha = {
        body: z.object({
            username: z.string(),
            codigo: z.string().min(1).max(255),
            senha: z.string().min(1).max(255).superRefine(validarForcaSenha)
        })
    };

    static solicitarRecuperacaoSenha = {
        body: z.object({
            email: z.string().email(),
            link_recuperacao: z.string().url()
        })
    };

    static confirmarEmail = {
        query: z.object({
            username: UsuarioSchemas.cadastrarUsuario.body.shape.username,
            codigo: z.string().min(1).max(255)
        })
    };

    static _tokenPayload = z.object({
        username: z.string(),
        email_confirmado: z.boolean()
    });
}