import { z } from "zod";
import { LINGUAGENS } from "../services/ExecAvaliar.js";
import { commonSchemas } from "../utils/zod.js";
import { codigoStatus, codigoTesteResultado } from "../models/Codigo.js";

export class CodigoSchemas {
    static enviarCodigo = {
        params: z.object({
            sigla: z.string().min(1).max(255)
        }),
        body: z.object({
            codigo: z.string().min(1).max(64000), // 64KB?
            linguagem: z.enum(LINGUAGENS)
        }).strict()
    };

    static codigoPorId = commonSchemas.getPorId();

    static meusCodigos = {
        query: commonSchemas.listagemPaginada()
    };

    static codigoGET = z.object({
        _id: z.string(),
        __v: z.number(),
        codigo: z.string(),
        problema: z.string(),
        linguagem: z.string(),
        status: z.enum(codigoStatus),
        data_envio: z.string(),
        testes: z.array(z.object({
            _id: z.string(),
            num: z.number(),
            saida: z.string().optional(),
            resultado: z.enum(codigoTesteResultado),
            argumentos: z.array(z.string()).optional(),
            distancia: z.number().optional(),
            tempo: z.number().optional(),
            data_processado: z.string().optional()
        })),
        usuario: z.string().optional(),
        data_recebido: z.string().optional(),
        data_avaliado: z.string().optional()
    });
}