import { z } from "zod";
import { commonSchemas } from "../utils/zod.js";

export class ProblemaSchemas {
    static _cadastrarProblemaTeste = z.object({
        argumentos: z.array(
            z.string().max(1024)
        ).max(128),
        saida: z.string().max(8192),
        regex: z.boolean().default(false)
    }).strict();

    static cadastrarProblema = {
        body: z.object({
            sigla: z.string().min(1).max(255).toLowerCase().optional(),
            titulo: z.string().min(1).max(512),
            descricao: z.string().min(1).max(64000),
            testes: z.array(this._cadastrarProblemaTeste)
                .min(1).max(64),
            testes_privados: z.array(this._cadastrarProblemaTeste)
                .max(64).default([])
        }).strict()
    };

    static listarProblemas = {
        query: commonSchemas.listagemPaginada()
    };

    static obterProblemaPelaSigla = {
        params: z.object({
            sigla: z.string().min(1).max(255).toLowerCase()
        }).strict()
    };

    static problemaGET = z.object({
        _id: z.string(),
        __v: z.number(),
        sigla: z.string(),
        titulo: z.string(),
        descricao: z.string(),
        descricao_html: z.string(),
        testes_quantidade: z.number(),
        testes: z.array(z.object({
            _id: z.string(),
            num: z.number(),
            argumentos: z.array(
                z.string()
            ),
            saida: z.string(),
            regex: z.boolean()
        }).strict()).min(1)
    }).strict();
}