import { z } from "zod";
import { commonSchemas } from "../utils/zod.js";
import { validarForcaSenha } from "../utils/forcaSenha.js";

export class UsuarioSchemas {

    static obterUsuario = {
        params: z.object({
            username: z.string()
        })
    };

    static cadastrarUsuario = {
        body: z.object({
            nome: z.string().min(1).max(255),
            username: z.string().min(3).max(32).regex(/^[a-zA-Z0-9_\-]+$/, "Nome de usuário inválido"),
            email: z.string().email(),
            senha: z.string().min(1).max(255).superRefine(validarForcaSenha)
        }).strict()
    };

    static banirUsuario = z.object({
        username: z.string(),
        banir_ate: z.string()
            .datetime({ message: "Formato de data inválido", local: true })
            .transform((str) => new Date(str))
            .nullable(),
    });

    static listarUsuarios = {
        query: commonSchemas.listagemPaginada().extend({
            username: z.string().optional()
        })
    };

    static usuarioGET = z.object({
        _id: z.string(),
        __v: z.number(),
        nome: z.string(),
        username: z.string(),
        email_confirmado: z.boolean(),
        banido_ate: z.string().nullable(),
        created_at: z.string(),
    }).strict();
}