import { ProblemaInteractor } from "../interactors/problemaInteractor.js";
import { ProblemaRepository } from "../models/Problema.js";

export default async function problemaSeed(repo: ProblemaRepository) {
    if (process.env.DEBUGLOG === "true")
        console.log("Fazendo seed de Problemas");

    let interactor = new ProblemaInteractor(repo);
    await repo.deleteEverything();

    await interactor.cadastrarProblema({
        titulo: "Soma de dois números",
        descricao: "<p>Faça um programa que receba dois números como argumentos e retorne a soma deles.</p>",
        testes: [
            {
                argumentos: ["1", "2"],
                saida: "3\n",
                regex: false
            },
            {
                argumentos: ["3", "4"],
                saida: "7\n",
                regex: false
            },
            {
                argumentos: ["5", "6"],
                saida: "11\n",
                regex: false
            }
        ],
        testes_privados: []
    });

    await interactor.cadastrarProblema({
        titulo: "Fatorial de um número",
        descricao: "<p>Faça um programa que receba um número como argumento e retorne o fatorial dele.</p>",
        testes: [
            {
                num: 0,
                argumentos: ["5"],
                saida: "120\n",
                regex: false
            },
            {
                num: 1,
                argumentos: ["6"],
                saida: "720\n",
                regex: false
            },
            {
                num: 2,
                argumentos: ["7"],
                saida: "5040\n",
                regex: false
            }
        ],
        testes_privados: []
    });

    await interactor.cadastrarProblema({
        "titulo": "Diferença entre datas",
        "descricao": `
        <div style="max-width: 800px; margin: auto; background: white; padding: 20px; border-radius: 10px; box-shadow: 0 0 10px rgba(0, 0, 0, 0.1);">
            <h1 style="color: #333;">Validação de Datas</h1>
            <p>
                Faça um programa que receba dos argumentos o valor numérico do dia, mês e ano de uma data, e informe na tela se o valor é uma data válida ou não de acordo com as regras abaixo.
            </p>
            <p>
                Para uma data ser válida, ela deve seguir, no mínimo, as seguintes regras:
            </p>
            <ul style="list-style: none; padding: 0;">
                <li style="background: #e2e2e2; margin: 5px 0; padding: 10px; border-radius: 5px;">1 - O valor do dia, mês e ano não podem ser 0 e não podem ser negativos.</li>
                <li style="background: #e2e2e2; margin: 5px 0; padding: 10px; border-radius: 5px;">2 - O valor do mês não pode ser maior que 12.</li>
                <li style="background: #e2e2e2; margin: 5px 0; padding: 10px; border-radius: 5px;">3 - O valor do dia não pode ser maior que 31.</li>
                <li style="background: #e2e2e2; margin: 5px 0; padding: 10px; border-radius: 5px;">4 - Se o mês for 2 (Fevereiro), 4 (Abril), 6 (Junho), 9 (Setembro) ou 11 (Novembro), o valor do dia não pode ser maior que 30.</li>
                <li style="background: #e2e2e2; margin: 5px 0; padding: 10px; border-radius: 5px; font-style: italic; color: #555;">
                    5 (opcional) - Se o ano for bissexto, é válido um mês de fevereiro com 29 dias, caso contrário, deve ter 28 dias. Para determinar se um ano é bissexto ou não, geralmente seguimos as regras abaixo:
                    <ul style="list-style: none; padding: 0; margin: 0;">
                        <li style="background: #dcdcdc; margin: 5px 0; padding: 10px; border-radius: 5px;">A cada 4 anos (é divisível por 4), é um ano bissexto.</li>
                        <li style="background: #dcdcdc; margin: 5px 0; padding: 10px; border-radius: 5px;">Porém a cada 100 anos (divisível por 100), não é ano bissexto.</li>
                        <li style="background: #dcdcdc; margin: 5px 0; padding: 10px; border-radius: 5px;">E a cada 400 anos (divisível por 400), é ano bissexto. (As últimas regras prevalecem sobre as primeiras)</li>
                    </ul>
                </li>
            </ul>
            <p>
                Ao executar o programa, ele deve escrever 'Data válida.' quando for válida a data, e no caso dela ser inválida, escreva 'Data inválida.'
            </p>
        </div>
        `,
        "testes": [
            {
                "num": 0,
                "argumentos": ["01", "01", "2024"],
                "saida": "Data válida.\n",
                "regex": false
            },
            {
                "num": 1,
                "argumentos": ["00", "01", "2024"],
                "saida": "Data inválida.\n",
                "regex": false
            },
            {
                "num": 2,
                "argumentos": ["42", "01", "2024"],
                "saida": "Data inválida.\n",
                "regex": false
            }],
        "testes_privados": [
            {
                "num": 3,
                "argumentos": ["01", "13", "2024"],
                "saida": "Data inválida.\n",
                "regex": false
            },
            {
                "num": 4,
                "argumentos": ["31", "04", "2024"],
                "saida": "Data inválida.\n",
                "regex": false
            },
            {
                "num": 5,
                "argumentos": ["29", "02", "2024"],
                "saida": "Data válida.\n",
                "regex": false
            },
            {
                "num": 6,
                "argumentos": ["29", "02", "2025"],
                "saida": "Data inválida.\n",
                "regex": false
            }
        ]
    });
}
