// https://www.sohamkamani.com/nodejs/executing-shell-commands/
//import util from "util";
import child_process from "child_process";

//const exec = util.promisify(child_process.exec);

export async function bashExec(command: string, args: string[] = [], input: string | undefined = "", timeout = 60000, killSignal: NodeJS.Signals = "SIGKILL") {
    let tempoInicio = performance.now();
    try {
        const resultado = await new Promise<{code: number, output: string}>((resolve, reject) => {
            //if(process.env.DEBUGLOG === "true")
            //console.log("SPAWNED:",[command, ...args]);
            // https://nodejs.org/api/child_process.html#child_processspawncommand-args-options
            const spawned = child_process.spawn(command, args, {
                shell: false,
                timeout: timeout,
                killSignal: killSignal
            });

            let combinedOutput = "";
            let exitCode: number | NodeJS.Signals | null = null;

            spawned.on("spawn", () => {
                // quando abriu o processo
            });

            spawned.on("error", (err) => {
                reject(err);
            }); 

            spawned.stdout.on("data", (data: any) => {
                // recebeu saída de dados
                combinedOutput += data.toString();
            });

            spawned.stderr.on("data", (data: any) => {
                // recebeu saída de erro
                combinedOutput += data.toString();
            });

            spawned.on("exit", (code, signal) => {
                exitCode = code !== null ? code : signal;
                if(exitCode === 0) {
                    resolve({code: exitCode, output: combinedOutput});
                } else {
                    reject({code: exitCode, output: combinedOutput});
                }
            });

            if(input) {
                spawned.stdin.write(input);
                spawned.stdin.end();
            }
        });

        return {sucess: true, output: resultado.output, duration: performance.now() - tempoInicio};
    } catch (err: any) {
        console.error(err);
        return {sucess: false, output: ""+err.output, duration: performance.now() - tempoInicio};
    }
}

/*export async function _old_bashExec(command, timeout = 10000, killSignal = "SIGKILL") {
    let tempoInicio = performance.now();
    try {
        const { stdout, stderr } = await exec(command, { timeout: timeout, killSignal: killSignal });
        let saida = stdout || "";
        if (stderr) {
            saida += "\nSTDERR:\n" + stderr;
        }

        return {sucess: true, output: saida, duration: performance.now() - tempoInicio};
    } catch (err) {
        return {sucess: false, output: (err.stdout || "") + (err.stderr || ""), duration: performance.now() - tempoInicio, error: err};
    }
}*/

/*
// NÃO ESTÁ SENDO USADO
// AQUI APENAS PARA TESTAR DEPOIS
// https://www.sohamkamani.com/nodejs/executing-shell-commands/

import pty from "node-pty";


// https://github.com/nodejs/node/issues/31409
export async function bashExec(command = "", args = [], input = "", timeout = 10000, killSignal = "SIGKILL") {
    let tempoInicio = performance.now();
    let tempoFim = performance.now();
    try {
        const resultado = await new Promise((resolve, reject) => {
            const ptyProcess = pty.spawn(command, args, {
                cwd: process.cwd(),
                env: process.env
            });

            let combinedOutput = "";
            ptyProcess.onData((data) => {
                combinedOutput += data;
            });

            ptyProcess.onExit((status) => {
                tempoFim = performance.now();
                if(status.exitCode === 0) {
                    resolve({code: status.exitCode, output: combinedOutput});
                } else {
                    reject({code: status.exitCode, output: combinedOutput});
                }
            });

            if(input) {
                ptyProcess.write(input);
                ptyProcess.write("\n");
            }
        });

        return {sucess: true, output: resultado.output, duration: tempoFim - tempoInicio};
    } catch (err) {
        console.error(err);
        return {sucess: false, output: err.output, duration: tempoFim - tempoInicio};
    }
}
*/

//console.log(await bashExec("las -lhas"));