import anyAscii from "any-ascii";
import { closest, distance } from "fastest-levenshtein";
// A ideia é comparar o texto de forma a ignorar pequenas diferenças, como espaços a mais ou a menos, ou quebras de linha

// https://en.wikipedia.org/wiki/Approximate_string_matching
/**
 * https://en.wikipedia.org/wiki/Levenshtein_distance
 * 
 * Calcula a diferença entre duas strings. O retorno é 0.0 se são iguais e cada unidade representa uma 'edição' de diferença
 * 
 * A Distância de Levenshtein calcula o número mínimo de operações de edição (inserções, exclusões ou substituições de caracteres)
 * necessárias para transformar uma palavra na outra. 
 * 
 * Por exemplo, a distância de Levenshtein entre "kitten" e "sitting" é 3, indicando que são necessárias três operações para converter uma na outra.
 * @returns 
 */
export const semelhancaTexto = (output: string, saida: string, usarRegex = false) => {
    // se usando regex não dá para comparar similaridade
    if(usarRegex) {
        return new RegExp(saida).test(output) ? 0.0 : 100.0;
    }
    
    // Primeiro verifica se são exatamente iguais
    if(output === saida) {
        return 0.0;
    }

    // Então verifica se o tamanho é muito diferente antes de tentar comparar
    if(Math.abs(output.length - saida.length) >= 100) {
        return 100.0;
    }

    // Saídas grande demais não são verificadas semelhança, pois se não pode demorar muito
    /*
    https://www.npmjs.com/package/fastest-levenshtein
    I generated 500 pairs of strings with length N. I measured the ops/sec each library achieves to process all the given pairs. Higher is better.
    Test Target 	        N=4 	N=8 	N=16 	N=32 	N=64 	N=128 	N=256 	N=512 	N=1024
    fastest-levenshtein 	44423 	23702 	10764 	4595 	1049 	291.5 	86.64 	22.24 	5.473
    */
    if(output.length >= 1024 || saida.length >= 1024) {
        return 100.0;
    }

    // Se não são iguais, verifica se são parecidos com levenshtein distance
    // https://en.wikipedia.org/wiki/Levenshtein_distance
    // https://www.npmjs.com/package/fastest-levenshtein
    return distance(output, saida);
};

//console.log(semelhancaTexto("fast","fast"));
//console.log(semelhancaTexto("fast","this is fast"));