import jwt from "jsonwebtoken";
import { TokenPayload } from "../types/express/index.js";
import { APIError } from "../middlewares/errorHandler.js";

export const decodeJWTToken = (JWTToken: string): TokenPayload => {
    try {    
        const payload = jwt.verify(JWTToken, process.env.JWT_SECRET!, {
            issuer: "PANELA-AUTH"
        });
        return payload as TokenPayload;
    } catch (error) {
        // Se der erro, traduzir o erro para uma mensagem mais amigável
        if(error instanceof jwt.JsonWebTokenError) {
            if(error.message == "jwt must be provided") {
                throw new APIError("O token de autenticação não foi fornecido", 400);
            } else if (error.message == "jwt malformed" || error.message == "invalid signature" || error.message == "invalid token") {
                throw new APIError("O token de autenticação foi alterado ou é inválido", 400);
            } else if (error.message == "jwt expired") {
                throw new APIError("O token de autenticação expirou", 400);
            } else {    
                throw new APIError("Problemas ao decodificar o token JWT: "+(error.message || error), 400);
            }
        } else {
            throw error; // 500
        }
    }
}