import validator from "validator";
import { z } from "zod";

export const validarForcaSenha = (value: string, ctx: any) => {
    //Validação da senha
    //A ideia é que pode fazer senhas sem caracteres especiais só se for uma senha grande

    // abcdABCD1234 -> 24 + 10 + 10 + 10 = 54
    if(value.length < 8) {
        ctx.addIssue({
            code: z.ZodIssueCode.custom,
            message: "O mínimo são 8 caracteres"
        });
        return;
    }

    const forcaSenha = validator.isStrongPassword(value,{
        returnScore: true, 
        pointsPerUnique: 2, 
        pointsPerRepeat: 1, 
        pointsForContainingLower: 10, 
        pointsForContainingUpper: 10, 
        pointsForContainingNumber: 10, 
        pointsForContainingSymbol: 15 
    });

    const minimoForca = 40 + 10;

    if(forcaSenha < minimoForca) // mínimo de 8 caracteres únicos contendo minusculas, maiusculas, numeros e simbolos
    {
        ctx.addIssue({
            code: z.ZodIssueCode.custom,
            message: "A senha não atingiu o mínimo de força "+forcaSenha+"/"+minimoForca
        });
        return;
    }

    // Aceito
    return;
};