import mongoose from "mongoose";

// false caso queira usar método normal de paginação e não mongoose-paginate-v2
const usarMongoosePaginate = true;

const myCustomLabels = {
    totalDocs: "resultados",
    docs: "resposta",
    limit: "limite",
    page: "pagina",
    totalPages: "totalPaginas",
    pagingCounter: false,
    hasPrevPage: false,
    hasNextPage: false,
    prevPage: false,
    nextPage: false
};

export const filtrosAnd = (filtros: any[]) => {
    return filtros.length == 0 ? {} :
        filtros.length == 1 ? filtros[0] :
           /*else*/                 { $and: filtros };
};

export const filtrosOr = (filtros: any[]) => {
    return filtros.length == 0 ? {} :
        filtros.length == 1 ? filtros[0] :
           /*else*/                 { $or: filtros };
};

type PaginateFunc = (filtros: any, opcoes: any) => Promise<any>;

export const findPaginado = async (model: mongoose.Model<any> & {paginate: PaginateFunc}, filtros: any, pagina = 1, opcoes: any) => {
    opcoes = {
        ...{
            limite: 16,
            sort: { _id: -1 },
            populate: undefined,
            lean: false,
            select: undefined
        }, ...opcoes
    };

    if (usarMongoosePaginate) {
        return await model.paginate(filtros || {}, {
            sort: opcoes.sort,
            populate: opcoes.populate,
            select: opcoes.select,
            lean: opcoes.lean,
            page: pagina,
            limit: opcoes.limite,
            customLabels: myCustomLabels
        });
    } else {
        // DEPOIS: Prover uma forma de paginar sem ser lento mesmo com centenas de milhares de documentos
        // A ideia é que ao invés de pular com skip e então limit, irá utilizar o último documento como "Âncora"
        let listagem = model.find(filtros || {});

        // padrão ordenar por mais recentes primeiro
        listagem = listagem.sort(opcoes.sort);

        if (pagina > 1) {
            listagem = listagem.skip(opcoes.limite * (pagina - 1));
        }

        listagem = listagem.limit(opcoes.limite);

        if (opcoes.populate) {
            listagem = listagem.populate(opcoes.populate);
        }

        if (opcoes.select) {
            listagem = listagem.select(opcoes.select);
        }

        return {
            resposta: opcoes.lean ? await listagem.lean() : await listagem,
            pagina: pagina,
            limite: opcoes.limite
        };
    }
};