import sanitizeHtml from "sanitize-html";
//import * as htmlparser2 from "htmlparser2";

// Match HEX and RGB
const colorRegex = [/^#(0x)?[0-9a-f]+$/i, /^rgb\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*\)$/];

/**
 https://gist.github.com/davidhartsough/a125b5cef0721880e034e16b8899b842
	This RegEx is intended to accept nearly every valid URL, with a few exceptions; it intentionally excludes:
	- IP addresses
	- networks ranges, database/server addresses, ports
	- any transfer protocol that isn't hypertext
	- any characters that aren't standard latin a-z (A-Z) or 0-9
		(which means unicode symbols, like "Dingbats", and other language characters are not accepted)
	- http://localhost/
	- data:text/plain;charset=utf-8,OHAI
	- tel:+1234567890
*/
const urlRegex = /^(https?:\/\/)((?!-)(?!.*--)[a-zA-Z\-0-9]{1,63}(?<!-)\.)+[a-zA-Z]{2,63}(\/[^\s]*)?$/;

const transformTagsA: sanitizeHtml.Transformer = function(tagName, attribs) {
	let href: string;
	if(!urlRegex.test(attribs.href)) {
		href = "#invalido";
	} else {
		// DEPOIS: de alguma forma avisar que irá sair da página ao clicar?
		href = attribs.href;
	}

	return {
		tagName: "a",
		attribs: {
			href: href,
			target: "_blank",
			rel: "noopener noreferrer"
		}
	};
};

// Configurado para aceitar apenas o que o SunEditor vai produzir e ainda ser seguro
export const sanitizarHTML = (dirty: string): string => {
	return sanitizeHtml(dirty, {
		allowedTags: [
			"address", "article", "aside", "footer", "header", "h1", "h2", "h3", "h4",
			"h5", "h6", "hgroup", "main", "nav", "section", "blockquote", "dd", "div",
			"dl", "dt", "hr", "li", "main", "ol", "p", "pre",
			"ul", "a", "abbr", "b", "bdi", "bdo", "br", "cite", "code", "data", "dfn",
			"em", "i", "kbd", "mark", "q", "rb", "rp", "rt", "rtc", "ruby", "s", "samp",
			"small", "span", "strong", "sub", "sup", "time", "u", "var", "wbr", "caption",
			"col", "colgroup", "table", "tbody", "td", "tfoot", "th", "thead", "tr",
			"del"
		],
		disallowedTagsMode: "discard",
		allowedAttributes: {
			//"*": ["style"],
			a: ["href", "target", "rel"]
		},
		//allowedStyles: {
			//"*": {
				//"color": colorRegex,
				//"background-color": colorRegex,
				// Match any number with px, em, or %
				//"font-size": [/^\d+(?:px|em|%)$/],
				//"text-align": [/^left$/, /^right$/, /^center$/]
				
				// Para poder ter recuo
				//"margin-left": [/^\d+(?:px|em|%)$/]
			//}
		//},
		// URL schemes we permit
		allowedSchemes: ["http", "https"],
		allowProtocolRelative: false,
		enforceHtmlBoundary: false,
		parseStyleAttributes: false, // Não permitir style
		nestingLimit: 32, // Se tiver tudo isso de aninhamento, é um problema, ignorar o resto
		// https://www.npmjs.com/package/sanitize-html/v/1.18.2#transformations
		transformTags: {
			// Transformar url, garantir que está de acordo com regex
			"a": transformTagsA
		}
	});
};

export const extrairApenasTexto = (html: string): string => {
	/* https://www.npmjs.com/package/sanitize-html#what-if-i-dont-want-to-allow-any-tags
		"What if I don't want to allow any tags?"
		Also simple! Set allowedTags to [] and allowedAttributes to {}.
 	*/
	let resultado = sanitizeHtml(html, {
		allowedTags: [],
		allowedAttributes: {},
		enforceHtmlBoundary: false,
		parseStyleAttributes: true
	});
	// remover espaços em branco duplicados, quebras de linha, tab, etc...
	resultado = resultado.replace(/\s+/g," ");

	// NÃO! desse jeito pode passar entidades HTML como < e >.
	// https://github.com/fb55/htmlparser2
	// let resultado = "";
	// const parser = new htmlparser2.Parser({
	// 	ontext(section) {
	// 		if(!section) return;

	// 		const text = section.trim().replace(/\s+/g," ");
	// 		if(text.length <= 0) return;

	// 		if(resultado.length > 0) resultado += " ";
	// 		resultado += text;
	// 	}
	// });
	// parser.write(html);
	// parser.end();

	//console.log(resultado);

	return resultado;
};

export default sanitizarHTML;
