import { jest, describe, expect, test, afterAll, beforeAll } from "@jest/globals";
import mongoose from "mongoose";

import { CodigoInteractor } from "../../src/interactors/codigoInteractor.js";

import { conectarBanco, desconetarBanco } from "../../src/config/dbConnect.js";
import { ExecAvaliarLocal } from "../../src/services/ExecAvaliar.js";
import { Problema } from "../../src/models/Problema.js";
// Experimentando com mock do repository (Isso não faz muito sentido no final)
describe("CodigoInteractor", () => {
    let codigoId: mongoose.Types.ObjectId | undefined;
    let interactor: CodigoInteractor | undefined;

    beforeAll(async () => {
        await conectarBanco();
        interactor = new CodigoInteractor(
            new ExecAvaliarLocal()
        );
    });

    test("Teste enviar código para ser executado", async () => {
        let problema: Problema = {
            _id: new mongoose.Types.ObjectId(),
            sigla: "teste-"+Date.now(),
            titulo: "Hello",
            descricao: "Diga Hello World.",
            descricao_html: "Diga Hello World.",
            testes: [
                {
                    num: 0,
                    argumentos: [],
                    saida: "Hello World!\n",
                    regex: false
                }
            ],
            testes_privados: [],
            testes_quantidade: 1
        };
        let codigo = "console.log('Hello World!');";
        let linguagem = "javascript";

        let codigoObj = await interactor!.enviarCodigo(problema, codigo, linguagem);
        expect(codigoObj.status).toBe("Enviado");

        codigoId = codigoObj._id;
        expect(codigoId).toBeDefined();
    });

    test("Deve verificar que o código foi avaliado", async () => {
        let status = "Enviado";
        let repetiu = 0;
        do {
            let codigoObj = await interactor?.repo.findById(codigoId!);
            if(codigoObj && codigoObj.status !== undefined) {
                status = codigoObj.status;
            }
            repetiu++;

            // dorme 1 seg https://stackoverflow.com/questions/45478730/jest-react-testing-check-state-after-delay
            await new Promise((r) => setTimeout(r, 100));
        } while((status == "Enviado" || status == "Avaliando") && repetiu < 300);

        expect(status).toBe("Sucesso");
    }, 120000);

    afterAll(async () => {
        await desconetarBanco();
    });
});