import { jest, describe, expect, test, beforeAll } from "@jest/globals";
import { ExecAvaliar } from "../../src/services/ExecAvaliar.js";

let possuiDocker = await ExecAvaliar.verificarDockerDisponivel();
const toSkipOrNot = possuiDocker ? test : test.skip;

describe("ExecAvaliar", () => {
    let execAvaliar: ExecAvaliar | undefined = undefined;

    // Primeiro verifica se tem docker instalado
    toSkipOrNot("Preparação", async () => {
        execAvaliar = new ExecAvaliar();
    });

    async function _testeavaliar(linguagem: string, codigo: string, entradas: string[], saidafn: (fruta: string, index: number) => string) {         
        let out = await execAvaliar!.avaliarTestesCodigo(codigo, linguagem,
            entradas.map((fruta, index) => {
                return {
                    num: index,
                    argumentos: [fruta],
                    saida: saidafn(fruta, index),
                    regex: false
                };
            }),
            async (t) => {
                return t;
            }
        );
        try {
            expect(out.sucess).toBe(true);
            expect(out.testes).toBeInstanceOf(Array);
            expect(out.testes.length).toBe(entradas.length);
            for (let teste of out.testes) {
                expect(teste.resultado).toBe("Sucesso");
                expect(teste.distancia).toBe(0);
            }
        } catch(e) {
            console.error(out);
            throw e;
        }
    }

    let frutas = ["batata", "maçã", "uva"];

    toSkipOrNot("Deve executar código Java", async () => {
        let codigo = `
        class Main {
            public static void main(String[] args) {
                System.out.println("JAVA - Enviou: "+args[0]);
            }
        }
        `;

        await _testeavaliar("java", codigo, frutas, (fruta, index) => { return `JAVA - Enviou: ${fruta}\n`;});
    }, 30000);

    toSkipOrNot("Deve executar código JavaScript", async () => {
        let codigo = `
        console.log("JS - Enviou:",process.argv[2]);
        `;

        await _testeavaliar("javascript", codigo, frutas, (fruta, index) => { return `JS - Enviou: ${fruta}\n`;});
    }, 30000);

    toSkipOrNot("Deve executar código portugol", async () => {
        let codigo = `
        programa {
            funcao inicio() {
                cadeia arg
                leia(arg)
                escreva("PORTUGOL - Enviou: "+arg)
            }
        }
        `;

        await _testeavaliar("portugol", codigo, frutas, (fruta, index) => { return `PORTUGOL - Enviou: ${fruta}\n`;});
    }, 30000);

    toSkipOrNot("Deve executar código python", async () => {
        let codigo = 
        "import sys\n"+
        "print(\"PYTHON - Enviou:\",sys.argv[1])\n";

        await _testeavaliar("python", codigo, frutas, (fruta, index) => { return `PYTHON - Enviou: ${fruta}\n`; });
    }, 30000);

    // Depois ver questão do golang, go: creating work dir: mkdir /tmp/go-build441434666: read-only file system
    // https://github.com/golang/go/issues/8451
    toSkipOrNot("Deve executar código golang", async () => {
        let codigo = 
`package main

import (
    "fmt"
    "os"
)

func main() {
    fmt.Println("GOLANG - Enviou:", os.Args[1])
}`;

        await _testeavaliar("golang", codigo, frutas, (fruta, index) => { return `GOLANG - Enviou: ${fruta}\n`; });
    }, 30000);


    toSkipOrNot("Deve executar código golang #2", async () => {
        let codigo = 
`package main

import (
    "fmt"
    "os"
    "strconv"
)

func main() {
    count, err := strconv.Atoi(os.Args[1])
    if err != nil {
        fmt.Println("Error:", err)
        return
    }

    for i := 0; i < count; i++ {
		if i > 0 {
			fmt.Print(",")
		}
		fmt.Print(i)
	}
	fmt.Println()
}`;

        await _testeavaliar("golang", codigo, ["2","5","10","50"], (n, index) => { 
            return Array.from({ length: parseInt(n) }, (_, i) => i).join(",")+"\n";
        });
    }, 30000);
});