"use server";
import { ProblemaGet, ProblemaGetSchema, ProblemaPost, ProblemaPostSchema, ProblemaTesteGet } from "@/schemas/Problema";
import { APIError, APIResponse, IAPIError, apiFetch } from "@/utils/fetchApi";
import { revalidatePath } from "next/cache";
import { RedirectType, redirect } from "next/navigation";
import { verificaLogin } from "./authAction";

export async function submitProblema(prevState: unknown, formData: unknown) : Promise<IAPIError | undefined> {
    if(!(await verificaLogin())) {
        redirect("/auth/login", RedirectType.replace);
    }

    if(!formData || !(formData instanceof FormData)) {
        return new APIError("FormData não foi passado ou inválido", 400).toJSON();
    }

    // VALIDAÇÃO DE DADOS
    let testes: ProblemaPost["testes"][0][] = [];
    for(let i = 0; i < 128; i++) {
        let argumentos = formData.get(`teste.${i}.argumentos`);
        let saida = formData.get(`teste.${i}.saida`);
        let regex = formData.get(`teste.${i}.regex`);
        if(!argumentos || !saida) {
            break;
        }

        let saidastr = saida as string;
        if(!saidastr.endsWith("\n")) saidastr += "\n";

        testes.push({
            argumentos: (argumentos as string).split(" "),
            saida: saidastr,
            regex: ""+(regex as string) === "true"
        });
    }
    const result = ProblemaPostSchema.safeParse({
        sigla: formData.get("sigla") || undefined,
        titulo: formData.get("titulo"),
        descricao: formData.get("descricao"),
        testes: testes,
        testes_privados: []
    });
    
    if(!result.success) {
        let errors = [];
        for (const issue of result.error.issues) {
            errors.push({
                path: issue.path.join("."),
                message: issue.message
            });
        }
        console.error(errors);
        return new APIError(errors, 400).toJSON();
    }

    // Fazer requisição POST para o servidor
    let response: APIResponse<ProblemaGet> | undefined;
    try {
        response = await apiFetch(ProblemaGetSchema, "POST", "/problemas", result.data);
    } catch (error) {
        console.error(error);
        if(error instanceof APIError) return error.toJSON();

        return new APIError("Erro ao enviar dados para o servidor", 500).toJSON();
    }

    // INVALIDAÇÃO DE CACHE
    revalidatePath("/problemas");

    // Redireciona de volta
    // TEM QUE CHAMAR NO DEPOIS DO TRY CATCH PQ APARENTEMENTE O REDIRECT LANÇA UMA EXCEÇÃO PARA FUNCIONAR
    redirect(`/problemas/${response.data.sigla.toLowerCase()}`, RedirectType.replace);
}