"use client";
import { CodigoGet, CodigoGetSchema } from "@/schemas/Codigo";
import { ProblemaGet } from "@/schemas/Problema";
import { apiFetch } from "@/utils/fetchApi";
import { useEffect, useState } from "react";

const corStatus: Map<string,string> = new Map([
    ["Avaliando", "text-yellow-500"],
    ["Erro", "text-red-500"],
    ["Sucesso", "text-green-500"]
]);

const statusConfig: Map<string,{cor: string, emoji: string}> = new Map([
    ["Sucesso", {cor: "text-green-500", emoji: "✅"}],
    ["Erro", {cor: "text-red-500", emoji: "❗"}],
    ["Diferente", {cor: "text-red-500", emoji: "❌"}],
    ["Formatacao", {cor: "text-yellow-500", emoji: "📝"}],
    ["Timeout", {cor: "text-yellow-500", emoji: "⏰"}],
    ["Na Fila", {cor: "text-black-500", emoji: "⏳"}],
    ["Executando", {cor: "text-blue-500", emoji: "🔄"}],
]);

export default function ProgressoAvaliar({ problema, codigo: codigoProp }: Readonly<{ 
    problema: ProblemaGet,
    codigo: CodigoGet
}>) {
    const [codigo, setCodigo] = useState<CodigoGet>(codigoProp);

    useEffect(() => {
        console.log("UseEffect ProgressoAvaliar:", codigo.status);
        const timeout = setTimeout(async () => {
            if(codigo.status === "Sucesso" || codigo.status === "Erro") return;

            const { data } = await apiFetch(CodigoGetSchema, "GET", `/codigos/${codigo._id}`);

            // RECURSIVIDADE?
            setCodigo(data);
        }, 250);

        return () => clearTimeout(timeout);
    }, [codigo]);

    return (<>
        <b>Envio:</b>{codigo._id} <br/>
        <b>Status:</b>
        <span className={corStatus.get(""+(codigo?.status || "Avaliando")) || "text-black-500"}>
            {codigo.status || "Desconhecido"}
        </span>
        <br/>
        <ul>
        {Array(problema.testes_quantidade).fill(null).map((_: unknown, i: number) => {
            let resultadoTeste = codigo.testes.find(teste => teste.num === i);
            let status = resultadoTeste?.resultado || "Na Fila";
            return (
                <li key={i}>
                {i+" - "} 
                    <span className={statusConfig.get(status)?.cor || ""}>
                        {statusConfig.get(status)?.emoji || ""}
                        {` ${status}`} 
                        {resultadoTeste?.tempo && ` ${resultadoTeste.tempo.toFixed(2)}ms`} 
                        {resultadoTeste?.distancia && resultadoTeste.distancia !== 0 ? ` (distância: ${resultadoTeste.distancia})` : ""}
                    </span>
                </li>
            );
        })
        } 
        </ul>       
    </>);
}