"use client";
import MySunEditor from "@/components/MySunEditor";
import { ProblemaGet, ProblemaTesteGet } from "@/schemas/Problema";
import { Button } from "@/components/ui/button";
import { useState, useActionState } from "react";
import { submitProblema } from "@/actions/submitProblemaAction";

function UmTeste({ teste, num }: Readonly<{
    teste:ProblemaTesteGet,
    num: number
}>) {
    return (
        <tr className="bg-gray-50 border-b border-gray-300 h-full max-h-24">
            <td>
                <input name={`teste.${num}.argumentos`} type="text" defaultValue={teste?.argumentos.join(" ") || ""} className="flex-none w-160 border border-gray-300 rounded p-2" />
            </td>
            <td>
                <input name={`teste.${num}.saida`} type="text" defaultValue={teste?.saida || ""} className="flex-none w-160 border border-gray-300 rounded p-2" />
            </td>
        </tr>
    );
}

export default function EditarProblema({ problema }: Readonly<{ problema?: ProblemaGet | undefined }>) {
    const [formState, action] = useActionState(submitProblema, undefined);
    const [testes, setTestes] = useState<ProblemaTesteGet[]>(problema?.testes || []);

    function adicionarTeste() {
        setTestes([...testes, { _id: "", num: 0, argumentos: [], saida: "", regex: false}]);
    }

    return (<form action={action} className="p-4">
        <h2>
            <input name={"sigla"} type="hidden" value={problema?.sigla || ""} />
            <input name={"titulo"} type="text" defaultValue={problema?.titulo || ""} className="w-full border border-gray-300 rounded p-2" />
        </h2>
        <MySunEditor 
            initialContentHTML={problema?.descricao_html || "Descrição detalhada do problema..."} 
            autoFocus={true}
            name="descricao"
        />
        <table className="w-full max-w-md text-left">
            <thead>
                <tr className="border-b uppercase border-gray-900" >
                    <th>Entrada</th>
                    <th>Saída</th>
                </tr>
            </thead>
            <tbody>
                {
                    (testes && testes.length > 0) && testes.map((teste, index) => {
                        return (<UmTeste teste={teste} num={index} key={index} />);
                    })
                }
                <tr>
                    <td colSpan={2}><Button onClick={adicionarTeste} type="button">Adicionar teste</Button></td>
                </tr>
            </tbody>
        </table>

        <Button type="submit">Enviar</Button>
    </form>);
}