import { Col } from "@/components/layout/RowCol";
import SunEditorHTMLViewer from "@/components/sunEditorHTMLViewer";
import { ProblemaGet, ProblemaTesteGet } from "@/schemas/Problema";
import Link from "next/link";

function UmTeste({ teste }: Readonly<{
    teste:ProblemaTesteGet
}>) {
    return (
        <tr className="w-full bg-gray-50 border-b border-gray-300 h-full max-h-24">
            <td>
                <pre>{teste.argumentos && Array.isArray(teste.argumentos) && teste.argumentos.join(" ")}</pre>
            </td>
            <td>
                <pre>{teste.saida}</pre>
            </td>
        </tr>
    );
}

export function DetalhesProblema({ problema, editavel }: Readonly<{ 
    problema: ProblemaGet
    editavel: boolean
}>) {
    return (<Col className="p-4">
        <h2>
            <b className="text-3xl">{problema.titulo}</b>
            {editavel && <Link href={`/problemas/${problema.sigla}/editar`} className="text-blue-500 p-2">Editar</Link>}
        </h2>
        <hr className="p-2"/>
        <SunEditorHTMLViewer contentHTML={problema.descricao_html} />
        <table className="w-full text-left">
            <thead>
                <tr className="border-b uppercase border-gray-900" >
                    <th>Entrada</th>
                    <th>Saída</th>
                </tr>
            </thead>
            <tbody className="w-full">
                {
                    problema.testes.length > 0 ? problema.testes.map((teste) => {
                        return (<UmTeste teste={teste} key={teste.num} />);
                    }) : <tr>
                        <td colSpan={2}>Nenhuma informação de testes disponível</td>
                    </tr>
                }
            </tbody>
        </table>
    </Col>);
}