"use client";
import { Button } from "@/components/ui/button";
import { apiFetch } from "@/utils/fetchApi";
import { CodigoGet, CodigoGetSchema, CodigoPostSchema } from "@/schemas/Codigo";
import { useRouter } from "next/navigation";

export default function FormularioProblema({ params }: Readonly<{ params: { sigla: string } }>) {
    const router = useRouter();
    async function onSubmit(event: React.FormEvent<HTMLFormElement>) {
        event.preventDefault();
        const formData = new FormData(event.currentTarget);

        const body = CodigoPostSchema.parse({
            codigo: formData.get("codigo"),
            linguagem: formData.get("linguagem")
        });

        // Faz POST
        const { data } = await apiFetch(CodigoGetSchema, "POST", `/problemas/${params.sigla}/codigo`, body);

        // Redireciona para página do status de avaliação
        router.push(`/codigos/${data._id}`);
    }

    return (<form className="h-full flex flex-col" onSubmit={onSubmit}>        
        <label htmlFor="linguagem" className="block">Linguagem</label>
        <select
            className="w-full border border-gray-300 rounded p-2"
            name="linguagem"
        >
            <option value="portugol">Portugol</option>
            <option value="javascript">Javascript</option>
            <option value="golang">Go</option>
            <option value="java">Java</option>
            <option value="python">Python</option>
        </select>
        <textarea
            name="codigo"
            className="w-full h-96 grow border border-gray-300 rounded p-2"
            placeholder="Digite seu código aqui..."
            defaultValue={`programa {
            funcao inicio() {
                inteiro a, b
                leia(a)
                leia(b)

                escreva(a + b)
            }
        }`}
        >
        </textarea>
        <Button>Enviar</Button>
    </form>);
}