import { z } from "zod";
import Link from "next/link";

import { ProblemaGetSchema } from "@/schemas/Problema";
import { apiFetch } from "@/utils/fetchApi";
import EstaLogado from "@/components/EstaLogado";
import { Button } from "@/components/ui/button";
import MainContainer from "@/components/MainContainer";
import { Col, Row } from "@/components/layout/RowCol";

export default async function Problemas() {
    // useEffect?
    const fetchResult = await apiFetch(z.array(ProblemaGetSchema), "GET", "/problemas");
    const problemas = fetchResult.data;
    
    if (problemas.length <= 0) {
        return <h2>Nenhum problema para exibir</h2>;
    }

    return (<MainContainer>
        <h2>
            <b>{problemas.length}</b> Problemas disponíveis! 
        </h2>

        <Col className="gap-2">
            {
                problemas.map(problema => {
                    return (<li
                        className="p-4 hover:bg-secondary hover:text-secondary-foreground"
                        key={problema._id}>
                            <Link href={`/problemas/${problema.sigla}`}>
                                <b className="text-xl">{problema.titulo}</b>
                            </Link>
                    </li>
                    );
                })
            }
            <EstaLogado se={<Button
                        className="p-4">
                <Link href="/problemas/novo/editar">Novo problema</Link>
            </Button>} />
        </Col>
    </MainContainer>);
}
