"use client";
import { cn } from "@/lib/utils";
import Link from "next/link";
import { usePathname } from "next/navigation";
import EstaLogadoClient from "./EstaLogadoClient";
import { Col, Row } from "./layout/RowCol";

export function Nav({ children }: Readonly<{
    children: React.ReactNode;
  }>) {
    return <nav className="bg-primary text-primary-foreground flex justify-center px-4">
        {children}
    </nav>;
}

export function NavLink(props: Record<string, string> & { 
    href: string; 
    pathName: string; 
    refQuery?: string;
    alignRight?: string; 
    className?: string; 
}) {
    const { href, refQuery, className, pathName, ..._props } = props;
    
    return <Link 
        {..._props} 
        href={refQuery === "true" ? { pathname: href, query: { ref: pathName } } : href}
        className={cn(`p-4 
        hover:bg-secondary hover:text-secondary-foreground
        focus-visible:bg-secondary focus-visible:text-secondary-foreground`,
        pathName === props.href && "bg-background text-foreground")} />;
}

export default function AppBar({ estaLogado }: Readonly<{ estaLogado: boolean }>) {
    const pathName = usePathname();
    return (<Row Elem={"header"} className="w-full shrink-0">
        <Row Elem={"nav"} className="w-full justify-center bg-primary text-primary-foreground">
            <NavLink pathName={pathName} href="/">Início</NavLink>
            <NavLink pathName={pathName} href="/problemas">Problemas</NavLink>
            <NavLink pathName={pathName} href={"/sobre"}>Sobre</NavLink>
            <NavLink pathName={pathName} href={"/teste"}>Teste</NavLink>
            { estaLogado && <NavLink pathName={pathName} href="/codigos">Meus Códigos</NavLink> }
            { estaLogado ? <NavLink pathName={pathName}  href="/auth/logout" refQuery={"true"}>
                    Sair
                </NavLink> 
                : <NavLink pathName={pathName}  href="/auth/login" refQuery={"true"}>
                    Entrar
                </NavLink>
            }
        </Row>         
    </Row>);
}
