"use client";
import dynamic from "next/dynamic";
import { useRef, useState } from "react";
import SunEditorCore from "suneditor/src/lib/core";
import "suneditor/dist/css/suneditor.min.css"; // Import Sun Editor's CSS File
import { Button } from "@/components/ui/button";
import SunEditorHTMLViewer from "@/components/sunEditorHTMLViewer";

function SunEditorLoader() {
    // 'Esqueleto' do editor enquanto ele carrega
    // (Sim eu copiei o html usando inspecionar no navegador e aí...)
    return <div className="sun-editor w-full">
        <div className="se-container">
            <div className="se-toolbar sun-editor-common min-h-12">
                <div className="se-btn-tray h-full">
                    <div className="se-btn-module se-btn-module-border h-full min-w-80">
                        <div className="se-menu-list">
                            <button type="button" className="se-btn se-tooltip" data-command="strike" data-display="" aria-label="Riscado">
                                ...
                            </button>
                        </div>
                    </div>
                   
                </div>
            </div>
            <div className="se-wrapper min-h-80">
                <div className="se-wrapper-inner se-wrapper-wysiwyg sun-editor-editable prose prose-slate max-w-none">
                    <p>...</p>
                </div>
            </div>
            <div className="se-resizing-bar sun-editor-common se-resizing-none">
                <div className="se-navigation sun-editor-common"></div>
            </div>
        </div>
    </div>;
}

// https://github.com/mkhstar/suneditor-react
const SunEditor = dynamic(() => import("suneditor-react"), {
    loading: () => <SunEditorLoader/>,
    ssr: false,
});

// Para ficar bonito o HTML produzido, renderizando com @tailwind/typography
// https://stackoverflow.com/questions/73547756/how-to-display-innerhtml-in-nextjs-having-tailwind
// https://github.com/tailwindlabs/tailwindcss-typography
export default function MySunEditor({ initialContentHTML, name, autoFocus }: Readonly<{ 
    initialContentHTML: string 
    name?: string
    autoFocus?: boolean
}>) {
    const [preview, setPreview] = useState<boolean>(false);
    const [content, setContent] = useState<string>(initialContentHTML);
    const editor = useRef<SunEditorCore>(undefined);

    // The sunEditor parameter will be set to the core suneditor instance when this function is called
    const getSunEditorInstance = (sunEditor: SunEditorCore) => {
        editor.current = sunEditor;
        //console.log(sunEditor.getContext().element.topArea);
    };

    const togglePreview = () => {
        if(!preview) {
            setContent(editor.current?.getContents(true) || "");
        }
        setPreview(!preview);
    };

    return (<>
            { !preview && <div className="sun-editor-wrapper">
                <SunEditor
                toggleCodeView={togglePreview}
                name={name || "editor"}
                autoFocus={autoFocus === true}
                getSunEditorInstance={getSunEditorInstance}
                setAllPlugins={true}
                lang={"pt_br"}
                setDefaultStyle={"height: auto; min-height: 20rem;"}
                setOptions={{
                    // https://github.com/JiHong88/SunEditor/#1-load-only-what-you-want
                    buttonList: [
                       ["formatBlock", "bold", "underline", "italic", "strike", "superscript", "removeFormat"],
                       ["link", "list", "blockquote", "horizontalRule", "table"],
                       ["codeView"]
                    ],
                    formats:  ["h1", "h2", "h3", "pre", "p"],
                    resizeEnable: true,
                    resizingBar: true,
                    showPathLabel: true,
                    className: "prose prose-slate max-w-none",
                }}
                defaultValue={content}/>
            </div> } { preview && <>
                <div className={"pb-6"}>
                    <Button onClick={() => togglePreview()}>Voltar para Edição</Button>
                </div>
                <SunEditorHTMLViewer contentHTML={content} />
            </>}
    </>);
}