import { z } from "zod";

export const ProblemaPostSchema = z.object({
    sigla: z.string().min(1).max(255).toUpperCase().optional(),
    titulo: z.string().min(1).max(512),
    descricao: z.string().min(1).max(64000),
    testes: z.array(z.object({
        argumentos: z.array(
            z.string().max(1024)
        ).max(128),
        saida: z.string().max(8192),
        regex: z.boolean().default(false)
    })).min(1).max(64),
    testes_privados: z.array(z.object({
        argumentos: z.array(
            z.string().max(1024)
        ).max(128),
        saida: z.string().max(8192),
        regex: z.boolean().default(false)
    })).max(64)
});
export const ProblemaGetSchema = z.object({
    _id: z.string(),
    __v: z.number(),
    sigla: z.string(),
    titulo: z.string(),
    descricao: z.string(),
    descricao_html: z.string(),
    testes_quantidade: z.number(),
    testes: z.array(z.object({
        _id: z.string(),
        num: z.number(),
        argumentos: z.array(z.string()),
        saida: z.string(),
        regex: z.boolean(),
    }))
});
export type ProblemaPost = z.infer<typeof ProblemaPostSchema>;
export type ProblemaGet = z.infer<typeof ProblemaGetSchema>;
export type ProblemaTesteGet = ProblemaGet["testes"][0];
// Tipo que define algo que é no mínimo um ProblemaPost com atributos ProblemaGet opcionais,
export type Problema = ProblemaPost & Partial<ProblemaGet>;